/* ---------------------------------------------------------------------------------------------------------------------
 * Copyright (c) AUMA Riester GmbH & Co. KG (2012). All Rights Reserved
 * Target System: SIMATIC PDM
 * ---------------------------------------------------------------------------------------------------------------------
 *  (c) Sipos Aktorik GmbH, 2017 (Mathias Monse <mathias.monse@sipos.de>)
 * ---------------------------------------------------------------------------------------------------------------------
 */
/* ----vvvvvvvv---- */
/* 0x0AC2: AC2_(1|2), 0x0C4F: AC2_1, 0x0CBD: AC2_2, 0x0C77: SXX */
/***
#define DEVICE_IDENT   0x0C77   
#define DEVICE_VERSION_MAJ   1  * 99: Base version *
#define DEVICE_VERSION_MIN   5
***/
/* ----^^^^^^^^---- */
#ifndef ID_CHECK_ABORT
#define	ID_CHECK_ABORT				0
#define	ID_CHECK_SUCCESS			1
#define	ID_CHECK_WRITABLE			0
#define	ID_CHECK_WRITE_PROTECTED	1
#endif
#ifndef ID_FC_ERROR
#define	ID_FC_ERROR					3000
#endif

#define FW_VERSION_DEFAULT      	99
#define IDENT_DEFAULT      		    0x0C77

#define FROM_FWVERSION_VALIDITY(_MAJ,_MIN)    IF (_MAJ <= var_version_major && _MIN <= var_version_minor ) { TRUE; } ELSE { FALSE; }
#define FROM_FWVERSION(_MAJ,_MIN, _X)    IF (_MAJ <= var_version_major && _MIN <= var_version_minor ) { _X }


#ifndef _V510_
#define LOG_MESSAGE(priority, text)  /* _TRACE(text)  */
#endif

#ifndef VARIABLE_STATUS_LOADED
#define VARIABLE_STATUS_LOADED        0x0010
#endif

/* ---------------------------------------------------------------------------------------------------------------------
 *   Defining VARIABLE  var_pb_ident (<=> PRM_3006) and COMMAND
 *   (use separate variable to avoid view object for access (compatibility issue with base version) 
 */
var_pb_ident LIKE VARIABLE PRM_3006 {}
ReadCmdDef(var_pb_ident, 30, 6)

/* ---------------------------------------------------------------------------------------------------------------------
 *   Defining VARIABLE  var_fw_version (<=> PRM_1759) and COMMAND
 *   (use separate variable to avoid view object for access (compatibility issue with base version) 
 */
var_fw_version LIKE VARIABLE  PRM_1759 {}
ReadCmdDef(var_fw_version, 17, 59)

/* --------------------------------------------------------------------------------------------------------------------- */
/*   Defining VARIABLE  var_version_major (internal use only) */
/*   */
VARIABLE var_version_major	
{
    LABEL		"fw version (major)";
    CLASS		LOCAL;
    TYPE		UNSIGNED_INTEGER (2);
    HANDLING		READ;
}
/* --------------------------------------------------------------------------------------------------------------------- */
/*   Defining VARIABLE  var_version_minor (internal use only) */
/*   */
VARIABLE var_version_minor	
{
    LABEL		"fw version (minor)";
    CLASS		LOCAL;
    TYPE		UNSIGNED_INTEGER (2);
    HANDLING		READ;
}

/* --------------------------------------------------------------------------------------------------------------------- */
/*   Defining Ident checking variables */
//
/* --------------------------------------------------------------------------------------------------------------------- */
/*   Defining VARIABLE  ret_CheckDeviceIdentification */
/*   */
VARIABLE	ret_CheckDeviceIdentification
{
	LABEL		"ret_CheckDeviceIdentification";
	CLASS		LOCAL;
	TYPE		UNSIGNED_INTEGER (1)
	{
		DEFAULT_VALUE	ID_CHECK_ABORT;
	}
	HANDLING	READ;
}
/* --------------------------------------------------------------------------------------------------------------------- */
/*   Defining VARIABLE  ret_CheckWriteProtection */
/*   */
VARIABLE	ret_CheckWriteProtection
{
	LABEL		"ret_CheckWriteProtection";
	CLASS		LOCAL;
	TYPE		UNSIGNED_INTEGER (1)
	{
		DEFAULT_VALUE	ID_CHECK_WRITABLE;
	}
	HANDLING	READ;
}

/* --------------------------------------------------------------------------------------------------------------------- */
/*   METHOD extract_version_majmin */
/*   */
/* INPAR in: version string from device variable (e.g. "01.05.02-1234") */
/* OUTPAR out1: extracted major version (e.g. "1") */
/* OUTPAR out2: extracted minor version (e.g. "5") */
//
/* TODO: parse and check version string: "[.+](\d\d)\.(\d\d)\.(\d\d)-(\d+)" */
/*       \1= major, \2= minor, \3= release, \4= build */
/*       and setup extracted version: "(\d\d)(\d\d)" */
/*       \1= major, \2= minor */
#ifndef MAX_VERSION_LENGTH
#define MAX_VERSION_LENGTH    4
#endif
METHOD extract_version_majmin (DD_STRING sinput, unsigned int & major, unsigned int & minor)
{
	LABEL		"extract_version_majmin";
	CLASS		INPUT;
	ACCESS		OFFLINE;
	DEFINITION
	{
		int i;
		int j;
		char vers[8];
		char in[20];
		in= sinput;
		vers= "";
		j= 0;
		for (i= 0; i < strlen(in) && j < MAX_VERSION_LENGTH; i++)
		{
			if (in[i] >= '0' && in[i] <= '9')
			{
				vers[j++]= in[i];
			}
		}
		if (strlen(vers) == 4)
		{
			major= atoi(vers)/100;
			minor= atoi(vers)%100;
		}
		else
		{
			major= 0;
			minor= 0;
		}
	}
}

/* --------------------------------------------------------------------------------------------------------------------- */
/*   METHOD build_version_string */
/*   */
/* INPAR in1: major version as unsigned int  */
/* INPAR in2: minor version as unsigned int  */
/* OUTPAR out1: version string (e.g. "03.01") */
//
METHOD build_version_string ( unsigned int major, unsigned int minor, DD_STRING &soutput )
{
	LABEL		"build_version_string";
	CLASS		INPUT;
	ACCESS		OFFLINE;
	DEFINITION
	{
		char sz[];
		
		if (major < 10) 
		{
			sz= "0";
		}
		sz += itoa(major);
		sz += ".";
		if (minor < 10) 
		{
			sz += "0";
		}
		sz += itoa(minor);
		soutput= sz;
	}
}

/* --------------------------------------------------------------------------------------------------------------------- */
/*   METHOD AumaDeviceIdent */
/*   */
/*   * Method reads Profibus-Ident (PRM_3006) und Firmware-Version (PRM_1759) and calculates  */
/*     ret_CheckWriteProtection and ret_CheckDeviceIdentification */
//
METHOD check_AUMADeviceIdent
{
	LABEL		"Identify AUMA device";
	CLASS		INPUT;
	ACCESS		OFFLINE;
	DEFINITION
	{
		int		iResult;
		int     required_pb_ident;
		int     required_fw_major;
		char    sz[];
		char    msg[];

#if defined (PA_IANDM)
		refresh_IM_record();
#endif
		/* Algorithm: */
		/* * Read PROFIBUS-Ident (PRM_3006) and fw version (PRM_1759) + extract Major-Minor-Version ("[<V>]<MM>.<mm>.<rr>-<bbbbb>" --> "<MM><mm>") */
		/* * check compatibility: */
		/* -- Ident_EDD == Ident_dev && required_fw_major <= var_version_major...  */
		/*    --> ident_ok + r/w */
		/* -- else --> no_ident + no_online */
		
		iResult = ID_FC_ERROR;
		required_pb_ident= DEVICE_IDENT;
		required_fw_major= DEVICE_VERSION_MAJ;

		/* LOG_MESSAGE(0, "### AumaDeviceIdent: read profibus ident"); */
		iResult = ReadCommand(read_var_pb_ident); /* read profibus ident (PRM_3006) */
		if (iResult >= ID_FC_ERROR)
		{
			/* LOG_MESSAGE(0, "### AumaDeviceIdent (ERROR):  Comm error while reading device ident"); */
			process_abort();
		}
		PRM_3006= var_pb_ident;
		PRM_3006.VARIABLE_STATUS= var_pb_ident.VARIABLE_STATUS;
		/* LOG_MESSAGE(0, "### AumaDeviceIdent: read firmware version"); */
		iResult = ReadCommand(read_var_fw_version); /* read f/w version (PRM_1759) */
		if (iResult >= ID_FC_ERROR)
		{
			/* LOG_MESSAGE(0, "### AumaDeviceIdent (ERROR):  Comm error while reading device version"); */
			process_abort();
		}
		PRM_1759= var_fw_version;
		PRM_1759.VARIABLE_STATUS= var_fw_version.VARIABLE_STATUS;
		/* workaround due to DPV1_Abort_Request */
		if ((var_pb_ident.VARIABLE_STATUS != VARIABLE_STATUS_LOADED) || (var_fw_version.VARIABLE_STATUS != VARIABLE_STATUS_LOADED))
		{
			/* device's check not possible yet  */
			/* LOG_MESSAGE(0, "### AumaDeviceIdent (result): device's check not possible yet --> write protected access \n" +  */
			/* 	"(var_pb_ident.VARIABLE_STATUS= " + var_pb_ident.VARIABLE_STATUS + ",\n var_fw_version.VARIABLE_STATUS= " + var_fw_version.VARIABLE_STATUS + ")"); */
			
			ret_CheckWriteProtection= ID_CHECK_WRITE_PROTECTED;
			ret_CheckDeviceIdentification= ID_CHECK_SUCCESS;
			return;
		}
		
		/* extract fw version */
		extract_version_majmin(var_fw_version, var_version_major, var_version_minor);

		/* check compatibility of device */
		/* LOG_MESSAGE(0, "### AumaDeviceIdent (read from device): ident= " + itoa(var_pb_ident) + ", version= " + var_fw_version); */
		if (var_pb_ident == required_pb_ident) {
			/* --> valid device  */
			/* LOG_MESSAGE(0, "### AumaDeviceIdent (result): device is valid --> writeable access"); */
			ret_CheckWriteProtection= ID_CHECK_WRITABLE;
			ret_CheckDeviceIdentification= ID_CHECK_SUCCESS;
			return;
		}
		else {
			/* otherwise no access permitted */
			/* LOG_MESSAGE(0, "### AumaDeviceIdent (result): no access permitted"); */
			get_dictionary_string(TXT_TST_1241, sz);
			msg= sz;
			get_dictionary_string(TXT_TST_1242, sz);
			msg += "\n" + sz;
			get_dictionary_string(TXT_TST_1238, sz);
			msg += "\n\n\t" + sz;
			build_version_string(var_version_major, var_version_minor, sz);
			msg += ":\t" + sz;
			get_dictionary_string(TXT_TST_1239, sz);
			msg += "\n\t" + sz;
			build_version_string(DEVICE_VERSION_MAJ, DEVICE_VERSION_MIN, sz);
			msg += ":\t" + sz;
			ACKNOWLEDGE(msg);

			ret_CheckWriteProtection= ID_CHECK_WRITE_PROTECTED;
			ret_CheckDeviceIdentification= ID_CHECK_ABORT;
			return;
		}
	}
}

/* --------------------------------------------------------------------------------------------------------------------- */
/*   Defining METHOD std_CheckWriteProtection */
/*   */
/*   Description: */
/*   * Method is to calculate ret_CheckWriteProtection */
/*     (1 - r/o, 0 - r/w) */
//
METHOD	std_CheckWriteProtection
{
	LABEL		"std_CheckWriteProtection";
	CLASS		INPUT;
	ACCESS		OFFLINE;
	DEFINITION
	{
		/* LOG_MESSAGE(0, "### std_CheckWriteProtection for AC2 devices is called"); */
		check_AUMADeviceIdent();
	}
}

/* --------------------------------------------------------------------------------------------------------------------- */
/*   Defining METHOD std_CheckDeviceIdentification */
/*   */
/*   Description: */
/*   * Method is called immediately before any online dialog is opened */
/*   * Method reads Profibus-Ident (PRM_3006) und Firmware-Version (PRM_1759) to calculate ret_CheckDeviceIdentification */
/*     (1 - device matches EDD, 0 - incompatible device) */
//
METHOD std_CheckDeviceIdentification
{
	LABEL		"std_CheckDeviceIdentification";
	CLASS		INPUT;
	ACCESS		OFFLINE;
	DEFINITION
	{
		/* LOG_MESSAGE(0, "### std_CheckDeviceIdentification for AC2 devices is called"); */
		check_AUMADeviceIdent();
	}
}

/* --------------------------------------------------------------------------------------------------------------------- */
/*   Defining METHOD std_CheckDeviceIdentificationDownload */
/*   */
/*   Description: */
/*   * Method is called immendiately before "Load to pc" is started */
/*   * Method reads Profibus-Ident (PRM_3006) und Firmware-Version (PRM_1759) to calculate ret_CheckDeviceIdentification */
/*     (1 - device matches EDD, 0 - incompatible device, no communication allowed) */
//
METHOD std_CheckDeviceIdentificationDownload
{
	LABEL		"std_CheckDeviceIdentificationDownload";
	CLASS		INPUT;
	ACCESS		OFFLINE;
	DEFINITION
	{
		/* LOG_MESSAGE(0, "### std_CheckDeviceIdentificationDownload for AC2 devices is called"); */
		check_AUMADeviceIdent();
	}
}

/* --------------------------------------------------------------------------------------------------------------------- */
/*   Defining METHOD std_CheckDeviceIdentificationUpload */
/*   */
/*   Description: */
/*   * Method is called immendiately before "Load to device" is started */
/*   * Method reads Profibus-Ident (PRM_3006) und Firmware-Version (PRM_1759) to calculate ret_CheckDeviceIdentification */
/*     (1 - device matches EDD, 0 - incompatible device, no communication allowed) */
//
METHOD std_CheckDeviceIdentificationUpload
{
	LABEL		"std_CheckDeviceIdentificationUpload";
	CLASS		INPUT;
	ACCESS		OFFLINE;
	DEFINITION
	{
		/* LOG_MESSAGE(0, "### std_CheckDeviceIdentificationUpload for AC2 devices is called"); */
		check_AUMADeviceIdent();
	}
}

/* --------------------------------------------------------------------------------------------------------------------- */
/*   End of file */
/* --------------------------------------------------------------------------------------------------------------------- */
