/*****************************************************************************
 *
 * File: SxC_def.inc
 * Version: 1.0.0
 * Date: 2017-05-18
 *
 *****************************************************************************
 * Copyright (c) 2017, AUMA Riester GmbH & Co. KG, All Rights Reserved
 *****************************************************************************
 *
 * Description: Electronic device description for PROFIBUS version of AUMATIC SXX
 *              (Common macro and constants definitions)
 *
 *****************************************************************************
 */
#ifdef _PDM_
#if __TOKVER__ < 700
// define constant for PDM6 interpreter specials
#define _PDM6_COMPAT_
#endif
#else
#ifndef dtmGENERATOR
#define _AMSDM_
#endif
#endif

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining Macro DeviceBoolVariable_RO: generate a device VARIABLE for device of boolean type
//     display as checkbox with LABEL text
//   - Arguments: 
//     _VAR - ID string of device variable (e.g. PZD_123)
//     _LEN - length in bytes (2 or 4)
#ifdef _PDM6_COMPAT_ // PDM6 compatibility mode

#define DeviceBoolVariable_RO(_VAR, _LEN)	VARIABLE _VAR\
{\
    LABEL	[TXT_ ## _VAR];\
    HELP	[HLP_ ## _VAR];\
    CLASS	INPUT & DYNAMIC;\
    TYPE	ENUMERATED (_LEN){{ 0, "false|de|nein" }, { 1, "true|de|ja" }}\
	HANDLING	READ;\
}

#else
#define DeviceBoolVariable_RO(_VAR, _LEN)	VARIABLE _VAR\
{\
    LABEL	[TXT_ ## _VAR];\
    HELP	[HLP_ ## _VAR];\
    CLASS	INPUT & DYNAMIC;\
    TYPE	BIT_ENUMERATED (_LEN){{ 1, [TXT_ ## _VAR] }}\
	HANDLING	READ;\
}
#endif
//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining Macro DeviceBoolDisplay: generate a menu item entry for boolean type variables
//   - Arguments: 
//     _VAR - ID string of device variable (e.g. PZD_123)
#ifdef _PDM6_COMPAT_ // PDM6 compatibility mode

#define DeviceBoolDisplay(_VAR)	_VAR

#else

#define DeviceBoolDisplay(_VAR)	_VAR (NO_LABEL)

#endif


//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining Macro LVN: generate a name for a local VARIABLE for device alert messages
//   - Arguments: 
//     _VAR - ID string of device variable (e.g. PZD_123)
//   --> Macro prefixes _VAR with "loc_"
#define LVN(_VAR)     loc_ ## _VAR

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining Macro L1VN: generate a name for a local VARIABLE for device alert messages
//   - Arguments: 
//     _VAR - ID string of device variable (e.g. PZD_123)
//   --> Macro prefixes _VAR with "loc1_"
#define L1VN(_VAR)     loc1_ ## _VAR

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining Macro DeviceAlertVariable: generate a device and a local VARIABLE for device alert messages
//   - Arguments: 
//     _VAR - ID string of device variable (e.g. PZD_123)
//     _LEN - length in bytes (2 or 4)
#ifdef _PDM6_COMPAT_ // PDM6 compatibility mode

#define DeviceAlertVariable(_VAR, _LEN)	VARIABLE _VAR\
{\
    LABEL	[TXT_ ## _VAR];\
    HELP	[HLP_ ## _VAR];\
    CLASS	INPUT & DYNAMIC;\
    TYPE	UNSIGNED_INTEGER (_LEN);\
	HANDLING	READ;\
	POST_READ_ACTIONS{DEFINITION{LVN(_VAR)=(_VAR != 0); LVN(_VAR).VARIABLE_STATUS= _VAR.VARIABLE_STATUS; save_values();}}\
}\
VARIABLE  L1VN(_VAR)\
{\
    LABEL	[TXT_ ## _VAR];\
    HELP	[HLP_ ## _VAR];\
    CLASS	LOCAL;\
    TYPE	ENUMERATED (_LEN){{ 0, "false|de|nein" }, { 1, "true|de|ja" }}\
	HANDLING	READ;\
}\
VARIABLE  LVN(_VAR)\
{\
    LABEL	[TXT_ ## _VAR];\
    HELP	[HLP_ ## _VAR];\
    CLASS	LOCAL;\
    TYPE	ENUMERATED (_LEN){{ 0, "false|de|nein" }, { 1, "true|de|ja" }}\
	HANDLING	READ;\
	VALIDITY IF(_VAR != 0) {TRUE;}ELSE{FALSE;}\
}

#else

#define DeviceAlertVariable(_VAR, _LEN)	VARIABLE _VAR\
{\
    LABEL	[TXT_ ## _VAR];\
    HELP	[HLP_ ## _VAR];\
    CLASS	INPUT & DYNAMIC;\
    TYPE	UNSIGNED_INTEGER (_LEN);\
	HANDLING	READ;\
	POST_READ_ACTIONS{DEFINITION{LVN(_VAR)=(_VAR != 0); LVN(_VAR).VARIABLE_STATUS= _VAR.VARIABLE_STATUS; \
	L1VN(_VAR)=(_VAR != 0); L1VN(_VAR).VARIABLE_STATUS= _VAR.VARIABLE_STATUS; save_values();}}\
}\
VARIABLE  L1VN(_VAR)\
{\
    LABEL	[TXT_ ## _VAR];\
    HELP	[HLP_ ## _VAR];\
    CLASS	LOCAL;\
    TYPE	BIT_ENUMERATED (_LEN){{ 1, [TXT_ ## _VAR] }}\
	HANDLING	READ;\
}\
VARIABLE  LVN(_VAR)\
{\
    LABEL	[TXT_ ## _VAR];\
    HELP	[HLP_ ## _VAR];\
    CLASS	LOCAL;\
    TYPE	BIT_ENUMERATED (_LEN){{ 1, [TXT_ ## _VAR] }}\
	HANDLING	READ;\
	VALIDITY IF(_VAR != 0) {TRUE;}ELSE{FALSE;}\
}

#endif

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining Macro DeviceAlertDisplay: generate a menu item entry for device alerts
//   - Arguments: 
//     _VAR - ID string of device variable (e.g. PZD_123)
#ifdef _PDM6_COMPAT_ // PDM6 compatibility mode

#define DeviceAlertDisplay(_VAR)    _VAR (HIDDEN), LVN(_VAR)

#else

#define DeviceAlertDisplay(_VAR)    _VAR (HIDDEN), LVN(_VAR) (NO_LABEL)

#endif

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining Macro DeviceAlertVariableCmp: generate a device and a local VARIABLE for device alert messages
//   (for compatibility reasons: depending on fw_major with 2 or 4 byte length)
//   - Arguments: 
//     _VAR - ID string of device variable (e.g. PZD_123)
#ifdef _PDM6_COMPAT_ // PDM6 compatibility mode

#define DeviceAlertVariableCmp(_VAR)	VARIABLE _VAR ## _1\
{\
    LABEL	[TXT_ ## _VAR];\
    CLASS	INPUT & DYNAMIC;\
    TYPE	UNSIGNED_INTEGER (2);\
	HANDLING	READ;\
	POST_READ_ACTIONS{DEFINITION{_VAR=(_VAR ## _1 != 0); _VAR.VARIABLE_STATUS= _VAR ## _1.VARIABLE_STATUS; save_values();}}\
	VALIDITY IF(var_version_major < 2){TRUE;}ELSE{FALSE;}\
}\
VARIABLE _VAR ## _2\
{\
    LABEL	[TXT_ ## _VAR];\
    CLASS	INPUT & DYNAMIC;\
    TYPE	UNSIGNED_INTEGER (4);\
	HANDLING	READ;\
	POST_READ_ACTIONS{DEFINITION{_VAR=(_VAR ## _2 != 0); _VAR.VARIABLE_STATUS= _VAR ## _2.VARIABLE_STATUS; save_values();}}\
	VALIDITY IF(var_version_major >= 2){TRUE;}ELSE{FALSE;}\
}\
VARIABLE  _VAR\
{\
    LABEL	[TXT_ ## _VAR];\
    HELP	[HLP_ ## _VAR];\
    CLASS	LOCAL;\
    TYPE	ENUMERATED (2){{ 0, "false|de|nein" }, { 1, "true|de|ja" }}\
	HANDLING	READ;\
	POST_WRITE_ACTIONS{DEFINITION{LVN(_VAR)=_VAR; LVN(_VAR).VARIABLE_STATUS= _VAR.VARIABLE_STATUS; save_values();}}\
}\
VARIABLE  LVN(_VAR)\
{\
    LABEL	[TXT_ ## _VAR];\
    HELP	[HLP_ ## _VAR];\
    CLASS	LOCAL;\
    TYPE	ENUMERATED (2){{ 0, "false|de|nein" }, { 1, "true|de|ja" }}\
	HANDLING	READ;\
	VALIDITY IF(_VAR != 0) {TRUE;}ELSE{FALSE;}\
}

#else

#define DeviceAlertVariableCmp(_VAR)	VARIABLE _VAR ## _1\
{\
    LABEL	[TXT_ ## _VAR];\
    CLASS	INPUT & DYNAMIC;\
    TYPE	UNSIGNED_INTEGER (2);\
	HANDLING	READ;\
	POST_READ_ACTIONS{DEFINITION{_VAR=(_VAR ## _1 != 0); _VAR.VARIABLE_STATUS= _VAR ## _1.VARIABLE_STATUS; save_values();}}\
	VALIDITY IF(var_version_major < 2){TRUE;}ELSE{FALSE;}\
}\
VARIABLE _VAR ## _2\
{\
    LABEL	[TXT_ ## _VAR];\
    CLASS	INPUT & DYNAMIC;\
    TYPE	UNSIGNED_INTEGER (4);\
	HANDLING	READ;\
	POST_READ_ACTIONS{DEFINITION{_VAR=(_VAR ## _2 != 0); _VAR.VARIABLE_STATUS= _VAR ## _2.VARIABLE_STATUS; save_values();}}\
	VALIDITY IF(var_version_major >= 2){TRUE;}ELSE{FALSE;}\
}\
VARIABLE  _VAR\
{\
    LABEL	[TXT_ ## _VAR];\
    HELP	[HLP_ ## _VAR];\
    CLASS	LOCAL;\
    TYPE	BIT_ENUMERATED (2){{ 1, [TXT_ ## _VAR] }}\
	HANDLING	READ;\
	POST_WRITE_ACTIONS{DEFINITION{LVN(_VAR)=_VAR; LVN(_VAR).VARIABLE_STATUS= _VAR.VARIABLE_STATUS; save_values();}}\
}\
VARIABLE  LVN(_VAR)\
{\
    LABEL	[TXT_ ## _VAR];\
    HELP	[HLP_ ## _VAR];\
    CLASS	LOCAL;\
    TYPE	BIT_ENUMERATED (2){{ 1, [TXT_ ## _VAR] }}\
	HANDLING	READ;\
	VALIDITY IF(_VAR != 0) {TRUE;}ELSE{FALSE;}\
}

#endif

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining Macro DeviceAlertAsBoolDisplay: generate a menu item entry for device alerts as bool
//   - Arguments: 
//     _VAR - ID string of device variable (e.g. PZD_123)
#ifdef _PDM6_COMPAT_ // PDM6 compatibility mode

#define DeviceAlertAsBoolDisplay(_VAR)    _VAR (HIDDEN), L1VN(_VAR)

#else

#define DeviceAlertAsBoolDisplay(_VAR)    _VAR (HIDDEN), L1VN(_VAR) (NO_LABEL)

#endif

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining Macro ReadCmdDef: generate a READ command definition
//   - Arguments: 
//     _VAR - ID string of device variable (e.g. PZD_123)
//     _SLT - Profibus slot
//     _IDX - Profibus index
#define ReadCmdDef(_VAR, _SLT, _IDX) COMMAND read_ ## _VAR\
{\
    SLOT		_SLT;\
    INDEX		_IDX;\
    OPERATION	READ;\
    TRANSACTION\
	{\
		REPLY { _VAR }\
        REQUEST {}\
    }\
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining Macro ReadCmdDefCmp: generate a READ command definition (using 2 variants of _VAR)
//   - Arguments: 
//     _VAR - ID string of device variable (e.g. PZD_123)
//     _SLT - Profibus slot
//     _IDX - Profibus index
#define ReadCmdDefCmp(_VAR, _SLT, _IDX) COMMAND read_ ## _VAR\
{\
    SLOT		_SLT;\
    INDEX		_IDX;\
    OPERATION	READ;\
    TRANSACTION\
	{\
		REPLY { _VAR ## _1, _VAR ## _2 }\
        REQUEST {}\
    }\
}

//---------------------------------------------------------------------------------------------------------------------
//~
//   Defining Macro WriteCmdDef: generate a WRITE command definition
//   - Arguments: 
//     _VAR - ID string of device variable (e.g. PZD_123)
//     _SLT - Profibus slot
//     _IDX - Profibus index
#define WriteCmdDef(_VAR, _SLT, _IDX) COMMAND write_ ## _VAR\
{\
    SLOT		_SLT;\
    INDEX		_IDX;\
    OPERATION	WRITE;\
    TRANSACTION\
	{\
		REPLY {}\
        REQUEST { _VAR }\
    }\
}

//--------------------------------------------------------------------------------------------------------------------
//~
//   Defining Macro LocalDevVar_Display: generate a menu item entry for VARIABLEs with a local derivative to display
//   - Arguments: 
//     _VAR - ID string of device variable (e.g. PZD_123)
#ifdef _PDM_
// functionality is not correct working with AMS (but with PDM it is)
#define LocalDevVar_Display(_VAR) _VAR (HIDDEN), ui ## _VAR
#else
#define LocalDevVar_Display(_VAR) _VAR
#endif

//--------------------------------------------------------------------------------------------------------------------
//~
//   Defining Macros for Min/Max Value dependencies of PRM_5114
#define MIN_VAL_BY_PRM_5114 SELECT (PRM_5114) {DEFAULT: 6;  CASE 1: 12;  CASE 2: 24; }
#define MAX_VAL_BY_PRM_5114 SELECT (PRM_5114) {DEFAULT: 60; CASE 1: 120; CASE 2: 240; }

//--------------------------------------------------------------------------------------------------------------------
//~
//   Defining Macros for Min/Max Value dependencies of PRM_5136
#define MIN_VAL_BY_PRM_5136 SELECT (PRM_5136) {DEFAULT: 42;  CASE 1: 120;  CASE 2: 240; }
#define MAX_VAL_BY_PRM_5136 SELECT (PRM_5136) {DEFAULT: 630; CASE 1: 1800; CASE 2: 3600; }

//---------------------------------------------------------------------------------------------------------------------
//~
//   End of file
//---------------------------------------------------------------------------------------------------------------------
